/*****************************************************************************/
/*                                                                           */
/*   Soft Power Switch with AVR                                              */
/*                                                                           */
/*   (c) 2008 Dipl.-Ing. (FH) Rainer Reusch, Reusch Elektronik for Elektor   */
/*   http://www.reusch-elektronik.de                                         */
/*   http://reweb.fh-weingarten.de/elektor                                   */
/*                                                                           */
/*   Atmel AVR Studio V4.13.528                                              */
/*   GNU C-Compiler GCC V4.2.20                                              */
/*                                                                           */
/*   Created: 2008-02-22                                                     */
/*   Changed: 2008-02-22                                                     */
/*                                                                           */
/*****************************************************************************/

// Preferences:
// CPU: ATmega8515 (applicable for all ATmega)
// Clock frequency: 9.216MHz (doesn't really care)
// Input from switch circuit at PB0
// Output to switch circuit at PB1
// Control LED output at PD4

#include <avr/interrupt.h>

// Port assignments
#define SwDir			DDRB	// data direction register
#define SwOutPort	PORTB	// output
#define SwInPort	PINB	// input
#define SwIn			PB0		// switch circuit input
#define SwOut			PB1		// switch circuit output
#define LEDDir		DDRD	// data direction register
#define LEDOut		PORTD	// output
#define LED				PD4		// output LED

// constants and variables
#define 	IRQS_PER_SECOND   1000									// generated timer 1 interrupts per second
#define 	TIME10MS					IRQS_PER_SECOND / 100	// counter value for 10ms
uint16_t 	TimerCount;															// counter for timer1 interrupts
uint16_t 	Timer10ms;                             	// counter for 10ms timer
uint16_t	MaxTime;																// used to set different LED blink rates (for demo)
volatile uint8_t State;														// power button state, possible values:
																									// 3 = just turned on, button still pressed
																									// 2 = keep on, button released
																									// 1 = button pressed for power off
																									// 0 = button released, now really turn off

//-----------------------------------------------------------------------------
void Init_Timer1(void)
//-----------------------------------------------------------------------------
// Init timer1 for IRQS_PER_SECOND 
{
  TCCR1A=0;	// Timer1: no PWM
  // Timer1 is counter: Clear Timer on Compare Match (CTC, Mode #4)
  // Timer1 is running with MCU clock / 64 (prescale=64)
	TCCR1B=(1<<WGM12)|(1<<CS11)|(1<<CS10);
  // OutputCompare for Timer1 frequency
  OCR1A = (unsigned short) ((unsigned long) F_CPU / 64 / IRQS_PER_SECOND -1);
  // activate OutputCompareA-Interrupt for Timer1
  TIMSK|=(1<<OCIE1A);
}

//-----------------------------------------------------------------------------
ISR(TIMER1_COMPA_vect)
//-----------------------------------------------------------------------------
{	// ISR for Timer1
	// 10ms timer
	Timer10ms++;
	if (Timer10ms>=TIME10MS)
	{	// 10ms passed
		Timer10ms=0;
		// check the power button
		switch (State)
		{
			case 3:
			case 1: if ((SwInPort&(1<<SwIn))>0) State--;	// change into next state, when button no longer pressed
			  break;
			case 2: if ((SwInPort&(1<<SwIn))==0) State--;	// change into next state, when button pressed again
			  break;
		}
	}
	MaxTime=IRQS_PER_SECOND>>State;	// set LED blink rate, depending on state (demo)
	// LED blink timer
	TimerCount++;
	if (TimerCount>=MaxTime)
	{	// time passed
		TimerCount=0;
		LEDOut^=(1<<LED);	// toggle LED
	}
}

//-----------------------------------------------------------------------------
int main(void)
//-----------------------------------------------------------------------------
{
	// initializations
	TimerCount=0;						// reset timer1 interrupt counter
	Timer10ms=0;      			// reset 10ms timer
	MaxTime=IRQS_PER_SECOND>>3;
	SwDir|=1<<SwOut;				// switch circuit output as output
	SwOutPort|=1<<SwOut;		// keep circuit on
	State=3;								// power on state
	LEDDir|=1<<LED;					// set LED port as output
	Init_Timer1();					// init timer1
	sei();									// enable interrupts

	while (State);					// main loop (do until turn off)

	// turn off
  TIMSK&=~(1<<OCIE1A);		// disable timer interrupt
	SwOutPort&=~(1<<SwOut);	// turn off circuit

	LEDOut|=(1<<LED);				// turn on LED for control (should turn off, because of no power)
	while (1);							// wait for real power off (prevents warning message)
}
